/*
 * Copyright 2016 Andrew Rucker Jones.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.opencsv.bean.mocks;

import com.opencsv.bean.CsvBindByName;
import com.opencsv.bean.CsvDate;

/**
 * Class for annotation test using {@link CsvDate} on a non-date field.
 * <p>Is also used incidentally in the following tests:<ul>
 * <li>Errors in input file with exceptions suppressed</li>
 * </ul></p>
 *
 * @author Andrew Rucker Jones
 */
public class DateAnnotationOnNondate {
    @CsvBindByName
    @CsvDate
    String isnotdate;

    public DateAnnotationOnNondate() {}
    public DateAnnotationOnNondate(String s) {isnotdate = s;}
}
